/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2009 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.api.web.gwt.client;

public abstract class SourceDecorator {
  public static final String CSS_CLASS_INDEX = "ln";
  public static final String CSS_CLASS_VALUE = "val";
  public static final String CSS_CLASS_SOURCE = "src";

  SourcePanel panel;

  public final void init(SourcePanel p) {
    this.panel = p;
    load(p.getResourceKey());
  }

  protected abstract void load(String resourceKey);


  protected final void onLoaded() {
    panel.onDecoratorLoaded();
  }

  public boolean hasValueCells() {
    return false;
  }

  public String decorateIndex(int lineIndex) {
    return "<div class='" + CSS_CLASS_INDEX + "'>" + lineIndex + "</div>";
  }

  public String decorateValue(int lineIndex) {
    return "<div class='" + CSS_CLASS_VALUE + "'>" + lineIndex + "</div>";
  }

  public String decorateSource(int lineIndex, String source) {
    return "<div class='" + CSS_CLASS_SOURCE + "'><pre>" + source + "</pre></div>";
  }

  public String decorateHeader() {
    return null;
  }
}
