/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2009 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.api.web.gwt.client;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.http.client.URL;
import com.google.gwt.json.client.JSONArray;
import com.google.gwt.json.client.JSONObject;
import com.google.gwt.json.client.JSONString;
import com.google.gwt.json.client.JSONValue;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.Panel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.widgetideas.table.client.PreloadedTable;

public class SourcePanel extends Composite {
  final Panel panel;
  final SourceDecorator decorator;
  boolean decoratorLoaded = false;
  final String resourceKey;
  List<String> sourceLines;
  LoadingLabel loading=null;

  public SourcePanel(String resourceKey, SourceDecorator decorator) {
    this.resourceKey = resourceKey;
    this.decorator = decorator;
    this.decorator.init(this);

    panel = new VerticalPanel();
    loading = new LoadingLabel();
    panel.add(loading);
    initWidget(panel);
    setStyleName("gwt-SourcePanel");
    requestSourceLines();
  }

  public String getResourceKey() {
    return resourceKey;
  }

  private void requestSourceLines() {
    String url = URL.encode(Utils.getServerUrl() + "/api/projects/" + resourceKey + "/sources?format=json&escapehtml=true&callback=");
    JsonUtils.requestJson(url, new SourceLinesHandler(this));
  }

  private void refresh() {
    if (sourceLines != null && decoratorLoaded) {
      PreloadedTable table = new PreloadedTable();
      table.setStyleName("sources");

      String headerHtml = decorator.decorateHeader();
      if (headerHtml!=null) {
        table.setPendingHTML(0, 2, headerHtml);
      }

      for (int index = 1; index < sourceLines.size() + 1; index++) {
        table.setPendingHTML(index, 0, decorator.decorateIndex(index));
        if (decorator.hasValueCells()) {
          table.setPendingHTML(index, 1, decorator.decorateValue(index));
        }
        table.setPendingHTML(index, 2, decorator.decorateSource(index, sourceLines.get(index - 1)));
      }

      loading.removeFromParent();
      panel.add(table);
    }
  }

  private void onSourceLinesLoaded(List<String> sourceLines) {
    this.sourceLines = sourceLines;
    refresh();
  }

  public void onDecoratorLoaded() {
    this.decoratorLoaded = true;
    refresh();
  }


  private static class SourceLinesHandler implements JsonUtils.JSONHandler {
    SourcePanel sourcePanel = null;

    private SourceLinesHandler(SourcePanel sourcePanel) {
      this.sourcePanel = sourcePanel;
    }

    public void handleJSON(JavaScriptObject obj) {
      if (obj == null) {
        Utils.showError("no JSON data");
        return;
      }
      List<String> sourceLines = new ArrayList<String>();
      JSONArray jsonArray = new JSONArray(obj);
      for (int i = 0; i < jsonArray.size(); i++) {
        JSONObject jsStock;
        JSONValue jsonValue;
        if ((jsStock = jsonArray.get(i).isObject()) == null) {
          continue;
        }
        if ((jsonValue = jsStock.get("lines")) != null) {
          JSONArray measures = jsonValue.isArray();
          for (int j = 0; j < measures.size(); j++) {
            JSONString value = null;
            if ((jsStock = measures.get(j).isObject()) == null) {
              continue;
            }
            if ((jsonValue = jsStock.get("value")) != null) {
              value = jsonValue.isString();
            }
            sourceLines.add(value != null ? value.stringValue() : "");
          }
        }
      }
      sourcePanel.onSourceLinesLoaded(sourceLines);
    }
  }
}

