#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
class Api::PropertiesController < Api::RestController

  require_role Role::ADMIN, :only => [ :create, :update, :destroy ]

  def index
    properties = []
    configuration.getDatabaseConfiguration.getKeys().each do |key|
      if viewable?(key)
        value = configuration.getProperty(key)
        property = Property.new
        property.prop_key = key
        property.prop_value = value
        properties << property
      end
    end
    rest_render(properties)
  end

  def show
    key = params[:id]
    conf_prop = configuration.getProperty(key)
    if conf_prop
      if viewable?(key)
        property = Property.new
        property.prop_key = key
        property.prop_value = conf_prop
        rest_render([property])
      else
        rest_status_ko('You are not authorized to see this ressource', 401)
      end
    else
      rest_status_ko('Property [' + params[:id] + '] does not exist', 404)
    end
  end

  def create
    key = params[:id]
    value = params[:value]
    if key
      begin
        configuration.setProperty(key, value)
        rest_status_ok
      rescue Exception => ex
        rest_status_ko(ex.message, 400)
      end
    else
      rest_status_ko('Property key [' + params[:id] + '] is not valid', 400)
    end
  end

  def update
    key = params[:id]
    value = params[:value]
    if key
      begin
        configuration.setProperty(key, value)
        rest_status_ok
      rescue Exception => ex
        rest_status_ko(ex.message, 400)
      end
    else
      rest_status_ko('Property key [' + params[:id] + '] is not valid', 400)
    end
  end

  def destroy
    key = params[:id]
    if key 
      begin
        configuration.clearProperty(key)
        rest_status_ok
      rescue Exception => ex
        rest_status_ko(ex.message, 400)
      end
    else
      rest_status_ko('Property key [' + params[:id] + '] is not valid', 400)
    end
  end

  protected

  def rest_to_json(properties)
    json = properties.collect{|property| property.to_hash_json}.to_json
    json = params['callback'] + '(' + json + ');' if params['callback']
    json
  end

  def rest_to_xml(properties)
    xml = Builder::XmlMarkup.new(:indent => 0)
    xml.instruct!
    xml.properties do
      properties.each do |property|
        xml << property.to_xml
      end
    end
  end

  private

  def viewable?(property_key)
    logged_in? or not property_key.to_s.index('.secured')
  end

end