#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
require 'java'

class ApplicationController < ActionController::Base
  
  include AuthenticatedSystem
  include RoleRequirementSystem

  before_filter :check_database

  def java_facade
    @java_facade ||= Java::OrgSonarCore::JRubyFacade.new
  end

  def configuration
    @configuration ||= java_facade.getConfiguration()
  end
  
  protected

  $operational_database = false

  def check_database
    if not $operational_database
      begin
        if load_current_database_version==load_expected_database_version
          $operational_database=true
        end
      rescue
      end
      if not $operational_database
        redirect_to :controller => 'setup', :action => 'index'
      end
    end
  end

  def load_current_database_version
    begin
      ActiveRecord::Migrator.current_version
    rescue
      -1
    end
  end

  def load_expected_database_version
    files = Dir["#{migrations_path}/[0-9]*_*.rb"].sort
    files.last.scan(/([0-9]+)_[_a-z0-9]*.rb/).first[0].to_i
  end

  def migrations_path
    File.dirname(__FILE__).to_s + "/../../db/migrate/"
  end



  # Do not log common errors like 404.
  # See http://maintainable.com/articles/rails_logging_tips
  EXCEPTIONS_NOT_LOGGED = ['ActionController::UnknownAction','ActionController::RoutingError']
  def log_error(exc)
    super unless EXCEPTIONS_NOT_LOGGED.include?(exc.class.name)
  end
end
