#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
begin
  require 'java'
  include_class 'org.sonar.core.Version'
rescue
  # Assuming we're under pure rails env for unit tests
end

module ApplicationHelper

  # hack for firefox. The 'alt' parameter on images does not work. Firefox uses 'title' instead.
  # http://snippets.dzone.com/posts/show/2917
  def image_tag(location, options={})
    options[:title] ||= options[:alt]
    super(location, options)
  end

  def sonar_version
    begin
      version = Version.getVersion()
    rescue
      # Assuming we're under pure rails env for unit tests
    end
    version || 'test-version'
  end

  def human_short_date(date, format = '%d.%m.%Y')
    case Date.today - date.to_date
    when  1 then 'yesterday'
    when  0 then date.strftime('%H:%M')
    else date.strftime(format)
    end
  end

  def human_date(date, format = '%b. %e, %Y')
    case Date.today - date.to_date
    when  1 then 'yesterday ' + date.strftime('%H:%M')
    when  0 then 'today ' + date.strftime('%H:%M')
    else date.strftime(format)
    end
  end

  def tendency_icon(measure, small=true, none_image='')
    suffix = (small ? '-small' : '')
    if small.nil? or measure.nil? or measure.tendency.nil? or measure.tendency==0
      return none_image
    end
    filename = measure.tendency.to_s
    
    case measure.tendency_qualitative
    when 0
      filename+= '-black'
    when -1
      filename+= '-red'
    when 1
      filename+= '-green'
    end
    image_tag("tendency/#{filename}#{suffix}.png")
  end

  def configuring?
    params['configuring']=='true'
  end

  def rules_bar_chart(image_id, mandatory_count, optional_count, max, size, include_optional=false)
    img_path = "/chart?cht=sb&chv=#{mandatory_count},#{optional_count},#{max - mandatory_count - optional_count}" +
      "&chc=777777,F8A036,FFFFFF&chdi=50x11&"

    if include_optional
      tooltip="#{mandatory_count.to_i + optional_count.to_i} violations (incl. #{optional_count.to_i} optional)"
    else
      tooltip="#{mandatory_count.to_i} violations"
    end
    image_tag(img_path, :size => size, :id => image_id, :style => 'vertical-align:middle', :alt => tooltip)
  end

  def bar_chart(image_id, values, colors, tooltip, size)
    img_path = "/chart?cht=sb&chv=#{values.join(',')}&chc=#{colors.join(',')}&chdi=55x11&"
    image_tag(img_path, :size => size, :id => image_id, :style => 'vertical-align:middle', :alt => tooltip)
  end


  def html_measure_to_drilldown(measure, metric_name=nil, show_alert_status=true, small=true)
    return '' unless measure
    url=url_for(:controller => 'breakdown', :action => 'measures', :id => measure.snapshot.project_id, :metric_id => measure.metric_id)
    html_measure(measure, metric_name, show_alert_status, url, nil, small)
  end

  def html_measure_to_violations_drilldown(measure, metric_name=nil, show_alert_status=true, small=true)
    return '' unless measure
    url=url_for :controller => 'breakdown', :action => 'rules', :id => measure.snapshot.project_id, :categ_id => nil
    html_measure(measure, metric_name, show_alert_status, url, nil, small)
  end

  def html_measure(measure, metric_name=nil, show_alert_status=true, url=nil, suffix='', small=true)
    html=''

    if measure and measure.metric
      link_rel=''
      show_link= !url.nil?

      if measure.metric.val_type==Metric::VALUE_TYPE_LEVEL
        html=image_tag("levels/#{measure.text_value.downcase}.png") unless measure.text_value.blank?
      else
        html=measure.formatted_value
      end

      alert_class=''
      if show_alert_status and not measure.alert_status.blank?
        alert_class="alert_#{measure.alert_status}" unless measure.metric.val_type==Metric::VALUE_TYPE_LEVEL
        link_rel=h(measure.alert_label)
        show_link=true
      end

      html="<span id='m_#{measure.metric.key}' class='#{alert_class}'>#{html}</span>"

      if show_link
        if url
          link_class=''
        else
          url='#'
          link_class='nolink'
        end
        html="<a href='#{url}' class='#{link_class}' rel='#{link_rel}' title='#{link_rel}'>#{html}</a>"
      end

      if metric_name
        html="#{html} #{metric_name}"
      end

      html="#{html} #{tendency_icon(measure, small)} #{suffix}"

    end
    html
  end
  
  def pos_value(number)
      number < 0 ? -number : number    
  end
end
