#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
class ActiveRule < ActiveRecord::Base
  belongs_to :rules_profile, :class_name => 'RulesProfile', :foreign_key => 'profile_id'
  belongs_to :rule
  has_many :active_rule_parameters, :dependent => :destroy

  def level
    failure_level
  end

  def error?
    failure_level==RuleFailure::LEVEL_ERROR
  end

  def warning?
    failure_level==RuleFailure::LEVEL_WARNING
  end

  def activated?
    error? or warning?
  end

  def value(param_id)
    active_rule_parameters.each do |param|
      return param.value if param.rules_parameter_id == param_id
    end
    nil
  end

  def parameters
    active_rule_parameters
  end

  def parameter(name)
    result=nil
    parameters.each do |param|
      result=param if (param.name==name)
    end
    result
  end

  def active_param_by_param_id(param_id)
    parameters.each do |param|
      return param if param.rules_parameter_id==param_id
    end
    nil
  end

  def copy
    new_active_rule = ActiveRule.new(:rule => rule, :failure_level => failure_level)
    self.active_rule_parameters.each do |active_rule_parameter|
      new_active_rule.active_rule_parameters << active_rule_parameter.copy
    end     
    new_active_rule 
  end
end
