 #
 # Sonar, entreprise quality control tool.
 # Copyright (C) 2007-2008 Hortis-GRC SA
 # mailto:be_agile HAT hortis DOT ch
 #
 # Sonar is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 3 of the License, or (at your option) any later version.
 #
 # Sonar is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with {library}; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 #
class ProjectLink < ActiveRecord::Base
  belongs_to :project
  
  LINK_HOMEPAGE = "homepage"
  LINK_CONTINUOUS_INTEGRATION = "ci"
  LINK_ISSUE_TRACKER = "issue"
  LINK_SCM_URL = "scm"
  LINK_SCM_RO_CONNECTION = "scm_ro"
  LINK_SCM_DEV_CONNECTION = "scm_dev"
  
  NAMES = {}
  NAMES[LINK_HOMEPAGE] = 'Homepage'
  NAMES[LINK_SCM_URL] = 'Sources (SCM)'
  NAMES[LINK_CONTINUOUS_INTEGRATION] = 'Continuous integration'
  NAMES[LINK_ISSUE_TRACKER] = 'Issue tracker'

  def display?
    name or ![nil, LINK_SCM_RO_CONNECTION, LINK_SCM_DEV_CONNECTION].include?(link_type)
  end
  
  def custom?
    link_type.nil?
  end
  
  def full_name
    if name
      name
    elsif link_type
      NAMES[link_type]
    else 
      ''
    end
  end
  
  def type
    link_type
  end

  def icon
    if custom?
      'links/external.png'
    else
      "links/#{link_type}.gif"
    end
  end

  def to_hash_json
    {'type' => link_type, 'name' => name, 'url' => href}
  end

  def to_xml
    xml = Builder::XmlMarkup.new
    xml.link do
      xml.type(link_type)
      xml.name(name)
      xml.url(href)
    end
  end

end