 #
 # Sonar, entreprise quality control tool.
 # Copyright (C) 2007-2008 Hortis-GRC SA
 # mailto:be_agile HAT hortis DOT ch
 #
 # Sonar is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
 # version 3 of the License, or (at your option) any later version.
 #
 # Sonar is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with {library}; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 #
class Property < ActiveRecord::Base

  self.primary_key = 'prop_key'
  
  TENDENCY_DEPTH = 'tendency.depth'
  
  def self.find_plugin_properties
    Property.find(:all, :conditions => ['prop_key LIKE ?','sonar.%'])
  end
  
  def key
    prop_key
  end
  
  def value
    prop_value
  end
  
  def validate_on_update
    errors.clear
    if (prop_key == TENDENCY_DEPTH)
      if prop_value.to_i <= 1
        errors.add('prop_value', "'#{prop_value}' is invalid. It must be a number greater than 1.")
      end
    end
  end
  
  def self.tendency_depth
    Property.find(:first, :conditions => {:prop_key => TENDENCY_DEPTH});
  end
  
  def self.update( prop_key, prop_value )
    property = Property.find(:first, :conditions => {:prop_key => prop_key});
    property.prop_value = prop_value
    property.save
    property
  end

  def to_hash_json
    {'key' => prop_key, 'value' => prop_value.to_s}
  end

  def to_xml(options={})
    xml = Builder::XmlMarkup.new
    xml.property do
      xml.key(prop_key)
      xml.value {xml.cdata!(prop_value.to_s)}
    end
  end

end
