#
# Sonar, entreprise quality control tool.
# Copyright (C) 2007-2008 Hortis-GRC SA
# mailto:be_agile HAT hortis DOT ch
#
# Sonar is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 3 of the License, or (at your option) any later version.
#
# Sonar is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with Sonar; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
#
require 'java'
include_class 'org.sonar.web.charts.jruby.ComplexityDistributionChart'
include_class 'org.sonar.web.charts.jruby.TrendsChart'

class Sonar::Jfreechart

  def complexity_chart(width, height, snapshot_ids, metric_key, label_suffix=nil)
    chart = ComplexityDistributionChart.new(width, height, ChartsController::CHART_COLORS, label_suffix)
    if snapshot_ids and snapshot_ids.size > 0
      snapshot_ids.each_with_index do |snapshot_id, index|
        measure = ProjectMeasure.find(:first, :conditions => {:metric_id => Metric::by_key(metric_key).id, :snapshot_id => snapshot_id})
        if measure and measure.text_value
          parts=measure.text_value.split(';')
          parts.each do |part|
            chart.addMeasure(index.to_i, part.split('=')[0] + '+', part.split('=')[1].to_i)
          end
        end
      end
      return export_chart_as_png(chart)    
    end
    nil
  end  


  def trends_chart(width, height, project_id, metrics_id, snapshots_id)

    @h_metric_categorynb = {}
    @i_category_count = -1
    
    # Separate metrics with and without category
    a_metrics_id_without_cat = []
    a_metrics_category_id = []
    metric_id_for_category = nil
    metrics_id.split(',').each do | metric_id_param |
      if !(metric_id_param.index('-').nil?)
        metric_id_for_category = metric_id_param.split('-').first
        a_metrics_category_id << metric_id_param.split('-').last
      else
        a_metrics_id_without_cat << metric_id_param
      end
    end
    
    # Init chart
    chart = TrendsChart.new(width, height)
    
    # Measures
    time_machine_add_measure(chart, time_machine_get_measures_without_category(project_id, a_metrics_id_without_cat*','))
    time_machine_add_measure(chart, time_machine_get_measures_with_category(project_id, metric_id_for_category, a_metrics_category_id*','))
    time_machine_add_measure(chart, time_machine_get_reviews(project_id, a_metrics_id_without_cat*','))    
    # Labels
    time_machine_add_labels(chart, project_id);

    export_chart_as_png(chart)
  end


  protected
  
  def time_machine_get_measures_with_category(project_id, metric_id, metric_category_id)
    if !metric_id.nil?
      sql = Project.send(:sanitize_sql, [
          "select s.created_at as created_at, m.value as value, m.rules_category_id as categ_id, m.metric_id as metric_id " +
            " from project_measures m LEFT OUTER JOIN snapshots s ON s.id=m.snapshot_id " +
            " where m.rule_id is null" +
            " and s.status='%s' " +
            " and s.project_id=%s " +
            " and m.metric_id=%s" + 
            " and m.rules_category_id in (%s)", Snapshot::STATUS_PROCESSED, project_id, metric_id, metric_category_id] )
      ProjectMeasure.connection.select_all( sql )
    end
  end

  def time_machine_get_measures_without_category(project_id, metric_id)
    if !metric_id.nil?
      sql = Project.send(:sanitize_sql, [
          "select s.created_at as created_at, m.value as value, m.rules_category_id as categ_id, m.metric_id as metric_id " +
            " from project_measures m LEFT OUTER JOIN snapshots s ON s.id=m.snapshot_id " +
            " where m.rule_id is null " +
            " and s.status='%s' " +
            " and s.project_id=%s " +
            " and m.metric_id in (%s) " + 
            " and m.rules_category_id is null ", Snapshot::STATUS_PROCESSED, project_id, metric_id] )
      ProjectMeasure.connection.select_all( sql )
    end
  end
  
  def time_machine_get_reviews(project_id, metric_id)
    if !metric_id.nil?
      sql = Project.send(:sanitize_sql, [
          "SELECT m.measure_date as created_at, m.value as value, m.rules_category_id as categ_id, m.metric_id as metric_id " +
            "FROM project_measures m " +
            "WHERE m.snapshot_id IS NULL " +
            "AND m.rule_id is null " +
            "AND m.project_id=%s " +
            "AND m.metric_id in (%s) " + 
            "AND m.rules_category_id is null ", project_id, metric_id] )
      ProjectMeasure.connection.select_all( sql )
    end    
  end  
  
  def time_machine_add_category(chart, metric_id, categ_id)
    key = metric_id
    key = key + '-' + categ_id if !categ_id.nil?
    
    if !@h_metric_categorynb[key]
      metric_obj = Metric.by_id(metric_id.to_i)
      @i_category_count = @i_category_count + 1
      @h_metric_categorynb[key] = @i_category_count

      category_number = @h_metric_categorynb[key]
      metric_category_id = categ_id

      if !categ_id.nil? and categ_id.length > 0
        category_obj = RulesCategory.by_id(metric_category_id.to_i)
        chart.initLegend(category_number, category_obj.name, metric_obj.val_type==Metric::VALUE_TYPE_PERCENT)
      else
        category_obj = Metric.by_id(metric_id.to_i)
        chart.initLegend(category_number, category_obj.short_name, metric_obj.val_type==Metric::VALUE_TYPE_PERCENT)
      end
    end

    return @h_metric_categorynb[metric_id]
  end

  def time_machine_add_measure(chart, sqlresult)
    if sqlresult and sqlresult.size>0
      sqlresult.each do |hash|
        time = Time.parse( hash["created_at"].to_s )
        value = hash["value"].to_f
        category_number = time_machine_add_category(chart, hash["metric_id"], hash["categ_id"])
        chart.addMeasure(value, time, category_number)
      end
    end
  end
  
  def time_machine_add_labels(chart, resource_id)
    Event.find(:all, :conditions => {:resource_id => resource_id}).each do |event|
      time = Time.parse( event.event_date.to_s )
      chart.addLabel(time, event.name, event.category==EventCategory::KEY_ALERT)
    end
  end
  
  def calculate_therical_value(percent, snapshot)
    nb_classes = snapshot.measure(Metric::NCSS_CLASSES).value
    return percent*nb_classes/100
  end  

  def export_chart_as_png(chart)
    java_int_array_to_bytes(chart.exportChartAsPNG())
  end

  def java_int_array_to_bytes( java_int_array )
    # pack does not exists on the return java array proxy object
    # the java proxy array contains integers, they must be packed
    # to return a correct byte array stream (c*)
    [].concat(java_int_array).pack("c*")
  end

end